@@ -26,6 +26,7 @@
 #include <linux/io.h>
 
 #include <mach/iomap.h>
+#include <mach/powergate.h>
 #include "board-paz00.h"
 
 #define PMC_CTRL                0x0
@@ -220,5 +221,7 @@ int __init paz00_power_init(void)
 		return -1;
 	}
 
+	tegra_powergate_power_off(TEGRA_POWERGATE_PCIE);
+
 	return 0;
 }
