@@ -155,6 +155,7 @@ static int qcnet_bind(struct usbnet *usbnet, struct usb_interface *iface)
 	struct usb_host_endpoint *endpoint = NULL;
 	struct usb_host_endpoint *in = NULL;
 	struct usb_host_endpoint *out = NULL;
+	int dir_in, dir_out, xfer_int;
 
 	if (iface->num_altsetting != 1) {
 		GOBI_ERROR("invalid num_altsetting %u", iface->num_altsetting);
@@ -168,24 +169,53 @@ static int qcnet_bind(struct usbnet *usbnet, struct usb_interface *iface)
 		return -EINVAL;
 	}
 
+	GOBI_DEBUG("interface number: %d",
+		iface->cur_altsetting->desc.bInterfaceNumber);
+
 	numends = iface->cur_altsetting->desc.bNumEndpoints;
 	for (i = 0; i < numends; i++) {
 		endpoint = iface->cur_altsetting->endpoint + i;
-		if (!endpoint) {
-			GOBI_ERROR("invalid endpoint %u", i);
-			return -EINVAL;
-		}
 
-		if (usb_endpoint_dir_in(&endpoint->desc)
-		&&  !usb_endpoint_xfer_int(&endpoint->desc)) {
+		dir_in   = usb_endpoint_dir_in(&endpoint->desc);
+		dir_out  = usb_endpoint_dir_out(&endpoint->desc);
+		xfer_int = usb_endpoint_xfer_int(&endpoint->desc);
+
+		GOBI_DEBUG("endpoint %d: addr %u "
+			"dir_in %d dir_out %d xfer_int %d",
+			i, endpoint->desc.bEndpointAddress,
+			dir_in, dir_out, xfer_int);
+
+		if (dir_in && !xfer_int) {
+			if (in) {
+				GOBI_ERROR("multiple in endpoints");
+				return -EINVAL;
+			}
+			GOBI_DEBUG("setting endpoint %d as in", i);
 			in = endpoint;
-		} else if (!usb_endpoint_dir_out(&endpoint->desc)) {
+		} else if (dir_out && !xfer_int) {
+			if (out) {
+				GOBI_ERROR("multiple out endpoints");
+				return -EINVAL;
+			}
+			GOBI_DEBUG("setting endpoint %d as out", i);
 			out = endpoint;
+		} else {
+			GOBI_DEBUG("ignoring endpoint %d", i);
 		}
 	}
 
 	if (!in || !out) {
-		GOBI_ERROR("invalid endpoints");
+		GOBI_ERROR("missing endpoint(s)");
+		if (in)
+			GOBI_ERROR("found in endpoint: %u",
+				in->desc.bEndpointAddress);
+		else
+			GOBI_ERROR("didn't find in endpoint");
+		if (out)
+			GOBI_ERROR("found out endpoint: %u",
+				out->desc.bEndpointAddress);
+		else
+			GOBI_ERROR("didn't find out endpoint");
 		return -EINVAL;
 	}
 
@@ -195,12 +225,14 @@ static int qcnet_bind(struct usbnet *usbnet, struct usb_interface *iface)
 		return -EINVAL;
 	}
 
-	usbnet->in = usb_rcvbulkpipe(usbnet->udev, in->desc.bEndpointAddress & USB_ENDPOINT_NUMBER_MASK);
-	usbnet->out = usb_sndbulkpipe(usbnet->udev, out->desc.bEndpointAddress & USB_ENDPOINT_NUMBER_MASK);
+	GOBI_DEBUG("found endpoints: in %u, out %u",
+		   in->desc.bEndpointAddress,
+		   out->desc.bEndpointAddress);
 
-	GOBI_DEBUG("in %x, out %x",
-	    in->desc.bEndpointAddress,
-	    out->desc.bEndpointAddress);
+	usbnet->in = usb_rcvbulkpipe(usbnet->udev,
+		in->desc.bEndpointAddress & USB_ENDPOINT_NUMBER_MASK);
+	usbnet->out = usb_sndbulkpipe(usbnet->udev,
+		out->desc.bEndpointAddress & USB_ENDPOINT_NUMBER_MASK);
 
 	return 0;
 }
@@ -209,6 +241,8 @@ static void qcnet_unbind(struct usbnet *usbnet, struct usb_interface *iface)
 {
 	struct qcusbnet *dev = (struct qcusbnet *)usbnet->data[0];
 
+	dev->dying = true;
+
 	iface->needs_remote_wakeup = 0;
 	netif_carrier_off(usbnet->net);
 	qc_deregister(dev);
@@ -273,6 +307,12 @@ static void qcnet_bg_startxmit(struct work_struct *work)
 	struct urb *urb = NULL;
 	int status;
 
+	if (dev->dying) {
+		GOBI_WARN("dying device");
+		/* Fear not; queued urbs will be freed in qcnet_disconnect. */
+		return;
+	}
+
 	if (dev->active)
 		return;
 
@@ -449,6 +489,91 @@ static u8 nibble(unsigned char c)
 	return 0;
 }
 
+static int discover_endpoints(struct qcusbnet *dev)
+{
+	int numends;
+	int i;
+	struct usb_host_endpoint *endpoint = NULL;
+	struct usb_host_endpoint *int_in   = NULL;
+	struct usb_host_endpoint *bulk_in  = NULL;
+	struct usb_host_endpoint *bulk_out = NULL;
+	int dir_in, dir_out, xfer_int;
+
+	GOBI_DEBUG("interface number: %d",
+		dev->iface->cur_altsetting->desc.bInterfaceNumber);
+
+	numends = dev->iface->cur_altsetting->desc.bNumEndpoints;
+	for (i = 0; i < numends; i++) {
+		endpoint = dev->iface->cur_altsetting->endpoint + i;
+
+		dir_in = usb_endpoint_dir_in(&endpoint->desc);
+		dir_out = usb_endpoint_dir_out(&endpoint->desc);
+		xfer_int = usb_endpoint_xfer_int(&endpoint->desc);
+
+		GOBI_DEBUG("endpoint %d: addr %x "
+			"dir_in %d dir_out %d xfer_int %d",
+			i, endpoint->desc.bEndpointAddress,
+			dir_in, dir_out, xfer_int);
+
+		if (dir_in && xfer_int) {
+			if (int_in) {
+				GOBI_ERROR("multiple int_in endpoints");
+				return -EINVAL;
+			}
+			GOBI_DEBUG("setting endpoint %d as int in", i);
+			int_in = endpoint;
+		} else if (dir_in && !xfer_int) {
+			if (bulk_in) {
+				GOBI_ERROR("multiple bulk_in endpoints");
+				return -EINVAL;
+			}
+			GOBI_DEBUG("setting endpoint %d as bulk in", i);
+			bulk_in = endpoint;
+		} else if (dir_out && !xfer_int) {
+			if (bulk_out) {
+				GOBI_ERROR("multiple bulk_out endpoints");
+				return -EINVAL;
+			}
+			GOBI_DEBUG("setting endpoint %d as bulk out", i);
+			bulk_out = endpoint;
+		} else {
+			GOBI_DEBUG("ignoring endpoint %d", i);
+		}
+	}
+
+	if (!int_in || !bulk_in || !bulk_out) {
+		GOBI_ERROR("missing endpoint(s)");
+		if (int_in)
+			GOBI_ERROR("found int_in endpoint: %u",
+				int_in->desc.bEndpointAddress);
+		else
+			GOBI_ERROR("didn't find int_in endpoint");
+		if (bulk_in)
+			GOBI_ERROR("found bulk_in endpoint: %u",
+				bulk_in->desc.bEndpointAddress);
+		else
+			GOBI_ERROR("didn't find bulk_in endpoint");
+		if (bulk_out)
+			GOBI_ERROR("found bulk_out endpoint: %u",
+				bulk_out->desc.bEndpointAddress);
+		else
+			GOBI_ERROR("didn't find bulk_out endpoint");
+		return -EINVAL;
+	}
+
+	dev->iface_num     = dev->iface->cur_altsetting->desc.bInterfaceNumber;
+	dev->int_in_endp   = int_in->desc.bEndpointAddress;
+	dev->bulk_in_endp  = bulk_in->desc.bEndpointAddress;
+	dev->bulk_out_endp = bulk_out->desc.bEndpointAddress;
+
+	GOBI_DEBUG("found endpoints: iface_num %u "
+		"int_in %u bulk_in %u bulk_out %u",
+		dev->iface_num,
+		dev->int_in_endp, dev->bulk_in_endp, dev->bulk_out_endp);
+
+	return 0;
+}
+
 int qcnet_probe(struct usb_interface *iface, const struct usb_device_id *vidpids)
 {
 	int status;
@@ -460,7 +585,7 @@ int qcnet_probe(struct usb_interface *iface, const struct usb_device_id *vidpids
 
 	status = usbnet_probe(iface, vidpids);
 	if (status < 0) {
-		GOBI_ERROR("usbnet_probe failed: %d", status);
+		GOBI_WARN("usbnet_probe failed: %d", status);
 		return status;
 	}
 
@@ -482,6 +607,16 @@ int qcnet_probe(struct usb_interface *iface, const struct usb_device_id *vidpids
 		return -ENOMEM;
 	}
 
+	dev->dying = false;
+	dev->iface = iface;
+
+	status = discover_endpoints(dev);
+	if (status) {
+		GOBI_ERROR("discover_endpoints failed: %d", status);
+		kfree(dev);
+		return status;
+	}
+
 	usbnet->data[0] = (unsigned long)dev;
 
 	dev->usbnet = usbnet;
@@ -505,7 +640,6 @@ int qcnet_probe(struct usb_interface *iface, const struct usb_device_id *vidpids
 
 	memset(&(dev->usbnet->net->stats), 0, sizeof(struct net_device_stats));
 
-	dev->iface = iface;
 	memset(&(dev->meid), '0', 14);
 
 	dev->valid = false;
