@@ -21,6 +21,8 @@
 #include "qcusbnet.h"
 
 #include <linux/compat.h>
+#include <linux/poll.h>
+#include <asm/byteorder.h>
 
 struct readreq {
 	struct list_head node;
@@ -43,6 +45,7 @@ struct client {
 	struct list_head reads;
 	struct list_head notifies;
 	struct list_head urbs;
+	wait_queue_head_t poll_queue;
 };
 
 struct urbsetup {
@@ -58,6 +61,8 @@ struct qmihandle {
 	struct qcusbnet *dev;
 };
 
+#define CID_NONE ((u16)-1)
+
 static int qcusbnet2k_fwdelay;
 
 static bool device_valid(struct qcusbnet *dev);
@@ -88,23 +93,24 @@ static ssize_t devqmi_read(struct file *file, char __user *buf, size_t size,
 			   loff_t *pos);
 static ssize_t devqmi_write(struct file *file, const char __user *buf,
 			    size_t size, loff_t *pos);
+static unsigned devqmi_poll(struct file *file,
+			    struct poll_table_struct *poll_table);
 
 static bool qmi_ready(struct qcusbnet *dev, u16 timeout);
 static void wds_callback(struct qcusbnet *dev, u16 cid, void *data);
 static int setup_wds_callback(struct qcusbnet *dev);
 static int qmidms_getmeid(struct qcusbnet *dev);
 
-#define IOCTL_QMI_GET_SERVICE_FILE      (0x8BE0 + 1)
-#define IOCTL_QMI_GET_DEVICE_VIDPID     (0x8BE0 + 2)
-#define IOCTL_QMI_GET_DEVICE_MEID       (0x8BE0 + 3)
-#define IOCTL_QMI_CLOSE		 (0x8BE0 + 4)
-#define CDC_GET_ENCAPSULATED_RESPONSE	0x01A1ll
-#define CDC_CONNECTION_SPEED_CHANGE	0x08000000002AA1ll
+#define IOCTL_QMI_GET_SERVICE_FILE	(0x8BE0 + 1)
+#define IOCTL_QMI_GET_DEVICE_VIDPID	(0x8BE0 + 2)
+#define IOCTL_QMI_GET_DEVICE_MEID	(0x8BE0 + 3)
+#define IOCTL_QMI_CLOSE		(0x8BE0 + 4)
 
 static const struct file_operations devqmi_fops = {
 	.owner   = THIS_MODULE,
 	.read    = devqmi_read,
 	.write   = devqmi_write,
+	.poll    = devqmi_poll,
 	.unlocked_ioctl   = devqmi_ioctl,
 #ifdef CONFIG_COMPAT
 	.compat_ioctl = devqmi_compat_ioctl,
@@ -236,6 +242,10 @@ static void read_callback(struct urb *urb)
 				break;
 			}
 
+			/* TODO(ttuttle): Should we do this only if a
+			   notify is not registered? */
+			wake_up_interruptible(&client->poll_queue);
+
 			notify = client_remove_notify(client, tid);
 			if (notify)
 				list_add(&notify->node, &notifies);
@@ -251,8 +261,13 @@ static void read_callback(struct urb *urb)
 
 static void int_callback(struct urb *urb)
 {
-	int status;
 	struct qcusbnet *dev = (struct qcusbnet *)urb->context;
+	int status;
+	int len;
+	u8 *buf;
+
+	static const u8 GET_ENCAPSULATED_RESPONSE = 0x01;
+	static const u8 CONNECTION_SPEED_CHANGE   = 0x2A;
 
 	if (!device_valid(dev)) {
 		GOBI_WARN("invalid device");
@@ -261,50 +276,88 @@ static void int_callback(struct urb *urb)
 
 	if (urb->status) {
 		GOBI_ERROR("urb status = %d", urb->status);
-		if (urb->status != -EOVERFLOW)
+		if (urb->status == -EOVERFLOW)
+			goto resubmit;
+		else
 			return;
-	} else {
-		if ((urb->actual_length == 8) &&
-		    (*(u64 *)urb->transfer_buffer == CDC_GET_ENCAPSULATED_RESPONSE)) {
-			usb_fill_control_urb(dev->qmi.readurb, dev->usbnet->udev,
-					     usb_rcvctrlpipe(dev->usbnet->udev, 0),
-					     (unsigned char *)dev->qmi.readsetup,
-					     dev->qmi.readbuf,
-					     DEFAULT_READ_URB_LENGTH,
-					     read_callback, dev);
-			status = usb_submit_urb(dev->qmi.readurb, GFP_ATOMIC);
-			if (status == 0) {
-				/* Do not resubmit the int_urb because
-				 * it will be resubmitted in read_callback */
-				return;
-			}
-			GOBI_ERROR("failed to submit read urb: %d", status);
-		} else if ((urb->actual_length == 16) &&
-			   (*(u64 *)urb->transfer_buffer == CDC_CONNECTION_SPEED_CHANGE)) {
-			/* if upstream or downstream is 0, stop traffic.
-			 * Otherwise resume it */
-			if ((*(u32 *)(urb->transfer_buffer + 8) == 0) ||
-			    (*(u32 *)(urb->transfer_buffer + 12) == 0)) {
-				qc_setdown(dev, DOWN_CDC_CONNECTION_SPEED);
-				GOBI_DEBUG("traffic stopping due to "
-					   "CONNECTION_SPEED_CHANGE");
-			} else {
-				qc_cleardown(dev, DOWN_CDC_CONNECTION_SPEED);
-				GOBI_DEBUG("resuming traffic due to "
-					   "CONNECTION_SPEED_CHANGE");
-			}
-		} else {
-			GOBI_ERROR("ignoring invalid int urb");
-			if (gobi_debug >= 1)
-				print_hex_dump(KERN_INFO, "gobi-int: ",
-					       DUMP_PREFIX_OFFSET, 16, 1,
-					       urb->transfer_buffer,
-					       urb->actual_length, true);
-		}
 	}
 
+	buf = urb->transfer_buffer;
+	len = urb->actual_length;
+
+	if (len < 8) {
+		GOBI_ERROR("urb too short (%d < 8)", len);
+		goto resubmit;
+	}
+
+	u8  req_type  = buf[0];
+	u8  request   = buf[1];
+	u16 iface_num = le16_to_cpup(buf + 4);
+
+	/* 0xA1 = dir: device-to-host, type: class, recipient: interface */
+	if (req_type != 0xA1) {
+		GOBI_ERROR("wrong request type (0x%02x != 0x%02x)",
+			req_type, 0xA1);
+		goto resubmit;
+	}
+
+	if (iface_num != dev->iface_num) {
+		GOBI_ERROR("wrong interface number (0x%04x != 0x%04x)",
+			iface_num, dev->iface_num);
+		goto resubmit;
+	}
+
+	if (request == GET_ENCAPSULATED_RESPONSE) {
+		if (len != 8) {
+			GOBI_ERROR("wrong length (%d != 8)", len);
+			goto resubmit;
+		}
+
+		GOBI_DEBUG("GET_ENCAPSULATED_RESPONSE");
+
+		usb_fill_control_urb(dev->qmi.readurb, dev->usbnet->udev,
+				     usb_rcvctrlpipe(dev->usbnet->udev, 0),
+				     (unsigned char *)dev->qmi.readsetup,
+				     dev->qmi.readbuf,
+				     DEFAULT_READ_URB_LENGTH,
+				     read_callback, dev);
+		status = usb_submit_urb(dev->qmi.readurb, GFP_ATOMIC);
+		if (status) {
+			GOBI_ERROR("failed to submit read urb: %d", status);
+			goto resubmit;
+		}
+		/* Do not resubmit the int_urb because
+		 * it will be resubmitted in read_callback */
+		return;
+	} else if (request == CONNECTION_SPEED_CHANGE) {
+		if (len != 16) {
+			GOBI_ERROR("wrong length (%d != 16)", len);
+			goto resubmit;
+		}
+
+		u32 upstream   = le32_to_cpup((__le32 *)(buf +  8));
+		u32 downstream = le32_to_cpup((__le32 *)(buf + 12));
+
+		GOBI_DEBUG("CONNECTION_SPEED_CHANGE: %d/%d",
+			upstream, downstream);
+
+		if (upstream == 0 || downstream == 0) {
+			qc_setdown(dev, DOWN_CDC_CONNECTION_SPEED);
+			GOBI_DEBUG("traffic stopping due to "
+				   "CONNECTION_SPEED_CHANGE");
+		} else {
+			qc_cleardown(dev, DOWN_CDC_CONNECTION_SPEED);
+			GOBI_DEBUG("resuming traffic due to "
+				   "CONNECTION_SPEED_CHANGE");
+		}
+		goto resubmit;
+	} else {
+		GOBI_ERROR("invalid request: 0x%02x", request);
+		goto resubmit;
+	}
+
+resubmit:
 	resubmit_int_urb(dev->qmi.inturb);
-	return;
 }
 
 int qc_startread(struct qcusbnet *dev)
@@ -360,13 +413,13 @@ int qc_startread(struct qcusbnet *dev)
 	dev->qmi.readsetup->type = 0xA1;
 	dev->qmi.readsetup->code = 1;
 	dev->qmi.readsetup->value = 0;
-	dev->qmi.readsetup->index = 0;
+	dev->qmi.readsetup->index = dev->iface_num;
 	dev->qmi.readsetup->len = DEFAULT_READ_URB_LENGTH;
 
 	interval = (dev->usbnet->udev->speed == USB_SPEED_HIGH) ? 7 : 3;
 
 	usb_fill_int_urb(dev->qmi.inturb, dev->usbnet->udev,
-			 usb_rcvintpipe(dev->usbnet->udev, 0x81),
+			 usb_rcvintpipe(dev->usbnet->udev, dev->int_in_endp),
 			 dev->qmi.intbuf, DEFAULT_READ_URB_LENGTH,
 			 int_callback, dev, interval);
 	status = usb_submit_urb(dev->qmi.inturb, GFP_KERNEL);
@@ -590,7 +643,7 @@ static int write_sync(struct qcusbnet *dev, struct buffer *data_buf, u16 cid)
 	ctx->setup.type = 0x21;
 	ctx->setup.code = 0;
 	ctx->setup.value = 0;
-	ctx->setup.index = 0;
+	ctx->setup.index = dev->iface_num;
 	ctx->setup.len = buffer_size(data_buf);
 
 	usb_fill_control_urb(urb, dev->usbnet->udev,
@@ -822,6 +875,7 @@ static int client_alloc(struct qcusbnet *dev, u8 type)
 	INIT_LIST_HEAD(&client->reads);
 	INIT_LIST_HEAD(&client->notifies);
 	INIT_LIST_HEAD(&client->urbs);
+	init_waitqueue_head(&client->poll_queue);
 
 	spin_unlock_irqrestore(&dev->qmi.clients_lock, flags);
 
@@ -863,6 +917,8 @@ static void client_free(struct qcusbnet *dev, u16 cid)
 		while (client_delread(dev, cid, 0, &data, &size))
 			kfree(data);
 
+		wake_up_all(&client->poll_queue);
+
 		list_del(&client->node);
 		kfree(client);
 		break;
@@ -1102,7 +1158,7 @@ static int devqmi_open(struct inode *inode, struct file *file)
 	}
 
 	handle = (struct qmihandle *)file->private_data;
-	handle->cid = (u16)-1;
+	handle->cid = CID_NONE;
 	handle->dev = ref;
 
 	GOBI_DEBUG("%p %04x", handle, handle->cid);
@@ -1138,7 +1194,7 @@ static long devqmi_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
 			return -EINVAL;
 		}
 
-		if (handle->cid != (u16)-1) {
+		if (handle->cid != CID_NONE) {
 			GOBI_WARN("cid already set");
 			return -EBADR;
 		}
@@ -1167,7 +1223,7 @@ static long devqmi_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
 	 * altogether).
 	 */
 	case IOCTL_QMI_CLOSE:
-		if (handle->cid == (u16)-1) {
+		if (handle->cid == CID_NONE) {
 			GOBI_WARN("no cid");
 			return -EBADR;
 		}
@@ -1242,7 +1298,7 @@ static int devqmi_release(struct inode *inode, struct file *file)
 
 	if (handle) {
 		file->private_data = NULL;
-		if (handle->cid != (u16)-1)
+		if (handle->cid != CID_NONE)
 			client_free(handle->dev, handle->cid);
 		qcusbnet_put(handle->dev);
 		kfree(handle);
@@ -1274,7 +1330,7 @@ static ssize_t devqmi_read(struct file *file, char __user *buf, size_t size,
 		return -ENXIO;
 	}
 
-	if (handle->cid == (u16)-1) {
+	if (handle->cid == CID_NONE) {
 		GOBI_WARN("cid is not set");
 		return -EBADR;
 	}
@@ -1323,7 +1379,7 @@ static ssize_t devqmi_write(struct file *file, const char __user * buf,
 		return -ENXIO;
 	}
 
-	if (handle->cid == (u16)-1) {
+	if (handle->cid == CID_NONE) {
 		GOBI_WARN("cid is not set");
 		return -EBADR;
 	}
@@ -1353,6 +1409,51 @@ static ssize_t devqmi_write(struct file *file, const char __user * buf,
 	return status;
 }
 
+static unsigned devqmi_poll(struct file *file,
+			    struct poll_table_struct *poll_table)
+{
+	struct qmihandle *handle = (struct qmihandle *)file->private_data;
+	struct client *client;
+	unsigned long flags;
+	unsigned status;
+
+	/* Always ready to write. */
+	status = POLLOUT | POLLWRNORM;
+
+	if (!handle) {
+		GOBI_WARN("handle is NULL");
+		return POLLERR;
+	}
+
+	if (!device_valid(handle->dev)) {
+		GOBI_WARN("invalid device");
+		return POLLERR;
+	}
+
+	if (handle->cid == CID_NONE) {
+		GOBI_WARN("cid is not set");
+		return POLLERR;
+	}
+
+	spin_lock_irqsave(&handle->dev->qmi.clients_lock, flags);
+
+	client = client_bycid(handle->dev, handle->cid);
+	if (!client) {
+		status = POLLERR;
+		goto out;
+	}
+
+	poll_wait(file, &client->poll_queue, poll_table);
+
+	if (!list_empty(&client->reads))
+		status |= POLLIN | POLLRDNORM;
+
+out:
+	spin_unlock_irqrestore(&handle->dev->qmi.clients_lock, flags);
+
+	return status;
+}
+
 int qc_register(struct qcusbnet *dev)
 {
 	int result;
@@ -1361,7 +1462,6 @@ int qc_register(struct qcusbnet *dev)
 	char *name;
 
 	dev->valid = true;
-	dev->dying = false;
 	result = client_alloc(dev, QMICTL);
 	if (result) {
 		dev->valid = false;
@@ -1430,7 +1530,6 @@ void qc_deregister(struct qcusbnet *dev)
 	struct list_head *node, *tmp;
 	struct client *client;
 
-	dev->dying = true;
 	list_for_each_safe(node, tmp, &dev->qmi.clients) {
 		client = list_entry(node, struct client, node);
 		client_free(dev, client->cid);
