@@ -28,6 +28,7 @@
 #include <linux/irq.h>
 #include <linux/list.h>
 #include <linux/mfd/core.h>
+#include <linux/module.h>
 #include <linux/mutex.h>
 #include <linux/notifier.h>
 #include <linux/platform_device.h>
@@ -140,7 +141,7 @@ static struct nvec_msg *nvec_msg_alloc(struct nvec_chip *nvec,
 
 	for (; i < NVEC_POOL_SIZE; i++) {
 		if (atomic_xchg(&nvec->msg_pool[i].used, 1) == 0) {
-			dev_vdbg(nvec->dev, "INFO: Alloc %i\n", i);
+			dev_vdbg(nvec->dev, "INFO: Allocate %i\n", i);
 			return &nvec->msg_pool[i];
 		}
 	}
@@ -436,6 +437,7 @@ static void nvec_rx_completed(struct nvec_chip *nvec)
 		/* Battery quirk - Often incomplete, and likes to crash */
 		if (nvec->rx->data[0] == NVEC_BAT)
 			complete(&nvec->ec_transfer);
+
 		return;
 	}
 
@@ -797,9 +799,7 @@ static int tegra_nvec_suspend(struct platform_device *pdev, pm_message_t state)
 {
 	struct nvec_chip *nvec = platform_get_drvdata(pdev);
 	struct nvec_msg *msg;
-
 	dev_dbg(nvec->dev, "suspending\n");
-	nvec_write_async(nvec, "\x0d\x10\x59\x94", 4);
 
 	/* keep these sync or you'll break suspend */
 	msg = nvec_write_sync(nvec, EC_DISABLE_EVENT_REPORTING, 3);
@@ -815,14 +815,12 @@ static int tegra_nvec_suspend(struct platform_device *pdev, pm_message_t state)
 static int tegra_nvec_resume(struct platform_device *pdev)
 {
 	struct nvec_chip *nvec = platform_get_drvdata(pdev);
-
 	dev_dbg(nvec->dev, "resuming\n");
 	clk_enable(nvec->clk);
 	enable_irq(nvec->irq);
 	/* when making the next line nvec_write_sync, resume seems to stop working */
 	nvec_write_async(nvec, EC_ENABLE_EVENT_REPORTING, 3);
 
-	nvec_write_async(nvec, "\x0d\x10\x59\x95", 4);
 	return 0;
 }
 
