@@ -624,6 +624,12 @@ unsigned long nvmap_handle_address(struct nvmap_client *c, unsigned long id)
 
 void nvmap_unpin(struct nvmap_client *client, struct nvmap_handle_ref *ref)
 {
+
+	WARN_ON(!ref);
+
+	if (!ref)
+		return;
+
 	atomic_dec(&ref->pin);
 	if (handle_unpin(client, ref->handle, false))
 		wake_up(&client->share->pin_wait);
@@ -762,6 +768,11 @@ EXPORT_SYMBOL_GPL(nvmap_alloc);
 
 void nvmap_free(struct nvmap_client *client, struct nvmap_handle_ref *r)
 {
+	WARN_ON(!r);
+
+	if (!r)
+		return;
+
 	nvmap_free_handle_id(client, nvmap_ref_to_id(r));
 }
 EXPORT_SYMBOL_GPL(nvmap_free);
